
module aux_rx
(
    output             O_frame_type  ,
    output             O_scrm_value  ,
    output reg         O_rx_end      ,
    output reg         O_rx_tps1_en  ,
    output reg         O_rx_tps2_en  ,
    input              I_aux_clk     ,
    input              I_aux_rstn    ,
    input      [3:0]   I_rx_cmd      ,  
    input      [19:0]  I_rx_addr     ,  
    input              I_rx_data_st  , 
    input      [7:0]   I_rx_data     ,        
    input              I_rx_data_vld ,        
    input      [7:0]   I_rx_len      , //actual +1
    input              I_rx_rply_st  , 
    output reg [7:0]   O_rx_rply_ack , 
    output reg [127:0] O_rx_rply_data,//high byte first {byte0,byte1,…,byte15} 
    output reg [7:0]   O_rx_rply_len , //actual +1
    output reg [7:0]   O_rx_rply_m  
    
); 

//===========================================================
localparam CMDTYPE_WR     = 4'b1000;//8
localparam CMDTYPE_RD     = 4'b1001;//9
localparam CMDTYPE_I2CWRM = 4'b0100;//4
localparam CMDTYPE_I2CRDM = 4'b0101;//5
localparam CMDTYPE_I2CWR  = 4'b0000;//0
localparam CMDTYPE_I2CRD  = 4'b0001;//1

localparam EDID_00 = 8'h00;
localparam EDID_01 = 8'hff;
localparam EDID_02 = 8'hff;
localparam EDID_03 = 8'hff;
localparam EDID_04 = 8'hff;
localparam EDID_05 = 8'hff;
localparam EDID_06 = 8'hff;
localparam EDID_07 = 8'h00;
localparam EDID_08 = 8'h30;
localparam EDID_09 = 8'hae;
localparam EDID_0A = 8'hc7;
localparam EDID_0B = 8'h62;
localparam EDID_0C = 8'h00;
localparam EDID_0D = 8'h00;
localparam EDID_0E = 8'h00;
localparam EDID_0F = 8'h00;
localparam EDID_10 = 8'h05;
localparam EDID_11 = 8'h21;
localparam EDID_12 = 8'h01;
localparam EDID_13 = 8'h04;
localparam EDID_14 = 8'ha5;
localparam EDID_15 = 8'h34;
localparam EDID_16 = 8'h20;
localparam EDID_17 = 8'h78;
localparam EDID_18 = 8'h3a;
localparam EDID_19 = 8'h2a;
localparam EDID_1A = 8'h15;
localparam EDID_1B = 8'haa;
localparam EDID_1C = 8'h54;
localparam EDID_1D = 8'h49;
localparam EDID_1E = 8'h9f;
localparam EDID_1F = 8'h24;
localparam EDID_20 = 8'h14;
localparam EDID_21 = 8'h50;
localparam EDID_22 = 8'h54;
localparam EDID_23 = 8'hbf;
localparam EDID_24 = 8'hef;
localparam EDID_25 = 8'h80;
localparam EDID_26 = 8'h71;
localparam EDID_27 = 8'h4f;
localparam EDID_28 = 8'h81;
localparam EDID_29 = 8'h8a;
localparam EDID_2A = 8'ha9;
localparam EDID_2B = 8'hcf;
localparam EDID_2C = 8'h81;
localparam EDID_2D = 8'h80;
localparam EDID_2E = 8'h95;
localparam EDID_2F = 8'h00;
localparam EDID_30 = 8'ha9;
localparam EDID_31 = 8'hc0;
localparam EDID_32 = 8'hb3;
localparam EDID_33 = 8'h00;
localparam EDID_34 = 8'hd1;
localparam EDID_35 = 8'hcf;
localparam EDID_36 = 8'h02;
localparam EDID_37 = 8'h3a;
localparam EDID_38 = 8'h80;
localparam EDID_39 = 8'h18;
localparam EDID_3A = 8'h71;
localparam EDID_3B = 8'h38;
localparam EDID_3C = 8'h2d;
localparam EDID_3D = 8'h40;
localparam EDID_3E = 8'h58;
localparam EDID_3F = 8'h2c;
localparam EDID_40 = 8'h45;
localparam EDID_41 = 8'h00;
localparam EDID_42 = 8'h0f;
localparam EDID_43 = 8'h28;
localparam EDID_44 = 8'h21;
localparam EDID_45 = 8'h00;
localparam EDID_46 = 8'h00;
localparam EDID_47 = 8'h1e;
localparam EDID_48 = 8'h00;
localparam EDID_49 = 8'h00;
localparam EDID_4A = 8'h00;
localparam EDID_4B = 8'hfd;
localparam EDID_4C = 8'h00;
localparam EDID_4D = 8'h32;
localparam EDID_4E = 8'h4b;
localparam EDID_4F = 8'h1e;
localparam EDID_50 = 8'h51;
localparam EDID_51 = 8'h12;
localparam EDID_52 = 8'h00;
localparam EDID_53 = 8'h0a;
localparam EDID_54 = 8'h20;
localparam EDID_55 = 8'h20;
localparam EDID_56 = 8'h20;
localparam EDID_57 = 8'h20;
localparam EDID_58 = 8'h20;
localparam EDID_59 = 8'h20;
localparam EDID_5A = 8'h00;
localparam EDID_5B = 8'h00;
localparam EDID_5C = 8'h00;
localparam EDID_5D = 8'hfc;
localparam EDID_5E = 8'h00;
localparam EDID_5F = 8'h45;
localparam EDID_60 = 8'h32;
localparam EDID_61 = 8'h34;
localparam EDID_62 = 8'h2d;
localparam EDID_63 = 8'h32;
localparam EDID_64 = 8'h38;
localparam EDID_65 = 8'h0a;
localparam EDID_66 = 8'h20;
localparam EDID_67 = 8'h20;
localparam EDID_68 = 8'h20;
localparam EDID_69 = 8'h20;
localparam EDID_6A = 8'h20;
localparam EDID_6B = 8'h20;
localparam EDID_6C = 8'h00;
localparam EDID_6D = 8'h00;
localparam EDID_6E = 8'h00;
localparam EDID_6F = 8'hff;
localparam EDID_70 = 8'h00;
localparam EDID_71 = 8'h56;
localparam EDID_72 = 8'h59;
localparam EDID_73 = 8'h43;
localparam EDID_74 = 8'h35;
localparam EDID_75 = 8'h35;
localparam EDID_76 = 8'h35;
localparam EDID_77 = 8'h31;
localparam EDID_78 = 8'h36;
localparam EDID_79 = 8'h0a;
localparam EDID_7A = 8'h20;
localparam EDID_7B = 8'h20;
localparam EDID_7C = 8'h20;
localparam EDID_7D = 8'h20;
localparam EDID_7E = 8'h01;
localparam EDID_7F = 8'hed;
localparam EDID_80 = 8'h02;
localparam EDID_81 = 8'h03;
localparam EDID_82 = 8'h08;
localparam EDID_83 = 8'hb1;
localparam EDID_84 = 8'h43;
localparam EDID_85 = 8'h90;
localparam EDID_86 = 8'h45;
localparam EDID_87 = 8'h04;
localparam EDID_88 = 8'h01;
localparam EDID_89 = 8'h1d;
localparam EDID_8A = 8'h00;
localparam EDID_8B = 8'h72;
localparam EDID_8C = 8'h51;
localparam EDID_8D = 8'hd0;
localparam EDID_8E = 8'h1e;
localparam EDID_8F = 8'h20;
localparam EDID_90 = 8'h6e;
localparam EDID_91 = 8'h28;
localparam EDID_92 = 8'h55;
localparam EDID_93 = 8'h00;
localparam EDID_94 = 8'h0f;
localparam EDID_95 = 8'h28;
localparam EDID_96 = 8'h21;
localparam EDID_97 = 8'h00;
localparam EDID_98 = 8'h00;
localparam EDID_99 = 8'h1e;
localparam EDID_9A = 8'h66;
localparam EDID_9B = 8'h21;
localparam EDID_9C = 8'h56;
localparam EDID_9D = 8'haa;
localparam EDID_9E = 8'h51;
localparam EDID_9F = 8'h00;
localparam EDID_A0 = 8'h1e;
localparam EDID_A1 = 8'h30;
localparam EDID_A2 = 8'h46;
localparam EDID_A3 = 8'h8f;
localparam EDID_A4 = 8'h33;
localparam EDID_A5 = 8'h00;
localparam EDID_A6 = 8'h0f;
localparam EDID_A7 = 8'h28;
localparam EDID_A8 = 8'h21;
localparam EDID_A9 = 8'h00;
localparam EDID_AA = 8'h00;
localparam EDID_AB = 8'h7f;
localparam EDID_AC = 8'hab;
localparam EDID_AD = 8'h22;
localparam EDID_AE = 8'ha0;
localparam EDID_AF = 8'ha0;
localparam EDID_B0 = 8'h50;
localparam EDID_B1 = 8'h84;
localparam EDID_B2 = 8'h1a;
localparam EDID_B3 = 8'h30;
localparam EDID_B4 = 8'h30;
localparam EDID_B5 = 8'h20;
localparam EDID_B6 = 8'h36;
localparam EDID_B7 = 8'h00;
localparam EDID_B8 = 8'h0f;
localparam EDID_B9 = 8'h28;
localparam EDID_BA = 8'h21;
localparam EDID_BB = 8'h00;
localparam EDID_BC = 8'h00;
localparam EDID_BD = 8'h7f;
localparam EDID_BE = 8'h7c;
localparam EDID_BF = 8'h2e;
localparam EDID_C0 = 8'h90;
localparam EDID_C1 = 8'ha0;
localparam EDID_C2 = 8'h60;
localparam EDID_C3 = 8'h1a;
localparam EDID_C4 = 8'h1e;
localparam EDID_C5 = 8'h40;
localparam EDID_C6 = 8'h30;
localparam EDID_C7 = 8'h20;
localparam EDID_C8 = 8'h36;
localparam EDID_C9 = 8'h00;
localparam EDID_CA = 8'h0f;
localparam EDID_CB = 8'h28;
localparam EDID_CC = 8'h21;
localparam EDID_CD = 8'h00;
localparam EDID_CE = 8'h00;
localparam EDID_CF = 8'h7f;
localparam EDID_D0 = 8'h00;
localparam EDID_D1 = 8'h00;
localparam EDID_D2 = 8'h00;
localparam EDID_D3 = 8'h00;
localparam EDID_D4 = 8'h00;
localparam EDID_D5 = 8'h00;
localparam EDID_D6 = 8'h00;
localparam EDID_D7 = 8'h00;
localparam EDID_D8 = 8'h00;
localparam EDID_D9 = 8'h00;
localparam EDID_DA = 8'h00;
localparam EDID_DB = 8'h00;
localparam EDID_DC = 8'h00;
localparam EDID_DD = 8'h00;
localparam EDID_DE = 8'h00;
localparam EDID_DF = 8'h00;
localparam EDID_E0 = 8'h00;
localparam EDID_E1 = 8'h00;
localparam EDID_E2 = 8'h00;
localparam EDID_E3 = 8'h00;
localparam EDID_E4 = 8'h00;
localparam EDID_E5 = 8'h00;
localparam EDID_E6 = 8'h00;
localparam EDID_E7 = 8'h00;
localparam EDID_E8 = 8'h00;
localparam EDID_E9 = 8'h00;
localparam EDID_EA = 8'h00;
localparam EDID_EB = 8'h00;
localparam EDID_EC = 8'h00;
localparam EDID_ED = 8'h00;
localparam EDID_EE = 8'h00;
localparam EDID_EF = 8'h00;
localparam EDID_F0 = 8'h00;
localparam EDID_F1 = 8'h00;
localparam EDID_F2 = 8'h00;
localparam EDID_F3 = 8'h00;
localparam EDID_F4 = 8'h00;
localparam EDID_F5 = 8'h00;
localparam EDID_F6 = 8'h00;
localparam EDID_F7 = 8'h00;
localparam EDID_F8 = 8'h00;
localparam EDID_F9 = 8'h00;
localparam EDID_FA = 8'h00;
localparam EDID_FB = 8'h00;
localparam EDID_FC = 8'h00;
localparam EDID_FD = 8'h00;
localparam EDID_FE = 8'h00;
localparam EDID_FF = 8'h3a;

//-------------------------------
//Read only registers
localparam RR00000 = 8'h12;//DPCD_REV[7:0], 0x12=DPCD r1.2
localparam RR00001 = 8'h06;//MAX_LINK_RATE[7:0], 0x14=5.4Gbps/lane (=0.27*RR00001)
localparam RR00002 = 8'h84;//MAX_LANE_COUNT[4:0], 0x04=4 lanes, ENHANCED_FRAME_CAP[7], 0=not support 
localparam RR00003 = 8'h00;//MAX_DOWNSPREAD[0], 1= down spread
localparam RR00004 = 8'h00;
localparam RR00005 = 8'h00;
localparam RR00006 = 8'h01;//MAIN_LINK_CHANNEL_CODING[0], 1=support 8b/10b
localparam RR00007 = 8'h00;//MSA_TIMING_PAR_IGNORED[6], 0=requires parameters, OUI Support[7], 0=not support OUI
localparam RR00008 = 8'h02;//LOCAL_EDID_PRESENT[1]=1, has a local EDID
localparam RR00009 = 8'h00;
localparam RR0000A = 8'h00;
localparam RR0000B = 8'h00;
localparam RR0000C = 8'h00;
localparam RR0000D = 8'h00;//eDP_CONFIGURATION_CAP, [0],1=eDP Device
localparam RR0000E = 8'h00;
localparam RR0000F = 8'h00;

localparam RR00020 = 8'h00;//RESERVED (FAUX-related register)
localparam RR00021 = 8'h00;//MST_CAP[0]=0=not support MST
localparam RR00022 = 8'h00;//NUMBER_OF_AUDIO_ENDPOINTS
localparam RR00023 = 8'h00;//AV_SYNC_DATA_BLOCK AV_GRANULARITY
localparam RR00024 = 8'h00;//00024h~0002Dh, AV_SYNC_DATA_BLOCK
localparam RR00025 = 8'h00;
localparam RR00026 = 8'h00;
localparam RR00027 = 8'h00;
localparam RR00028 = 8'h00;
localparam RR00029 = 8'h00;
localparam RR0002A = 8'h00;
localparam RR0002B = 8'h00;
localparam RR0002C = 8'h00;
localparam RR0002D = 8'h00;

localparam RR00070 = 8'h00;//DPCD r1.3 (eDP v1.4 DPRX) only, PANEL SELF REFRESH CAPABILITY SUPPORTED

localparam RR00080 = 8'h00;//00080h~0008Fh, For Wireless DFP
localparam RR00081 = 8'h00;
localparam RR00082 = 8'h00;
localparam RR00083 = 8'h00;
localparam RR00084 = 8'h00;
localparam RR00085 = 8'h00;
localparam RR00086 = 8'h00;
localparam RR00087 = 8'h00;
localparam RR00088 = 8'h00;
localparam RR00089 = 8'h00;
localparam RR0008A = 8'h00;
localparam RR0008B = 8'h00;
localparam RR0008C = 8'h00;
localparam RR0008D = 8'h00;
localparam RR0008E = 8'h00;
localparam RR0008F = 8'h00;

localparam RR00200 = 8'h41;//SINK_COUNT[7,5:0]=1, CP_READY[6]=1, support HDCP

localparam RR00400 = 8'h00;//00400h~00402h, Sink Device IEEE_OUI
localparam RR00401 = 8'hE0;
localparam RR00402 = 8'h4C;
localparam RR00403 = 8'h44;//00403h~00408h, Sink Device Identification String
localparam RR00404 = 8'h70;
localparam RR00405 = 8'h31;
localparam RR00406 = 8'h2E;
localparam RR00407 = 8'h32;
localparam RR00408 = 8'h00;
localparam RR00409 = 8'h00;//Sink Device Hardware Revision
localparam RR0040A = 8'h00;//Sink Device Firmware/Software Major Revision
localparam RR0040B = 8'h00;//Sink Device Firmware/Software Minor Revision

localparam RR00500 = 8'h00;//00500h~00502h, Branch Device IEEE_OUI
localparam RR00501 = 8'h00;
localparam RR00502 = 8'h00;
localparam RR00503 = 8'h00;//00503h~00508h, Branch Device Identification String
localparam RR00504 = 8'h00;
localparam RR00505 = 8'h00;
localparam RR00506 = 8'h00;
localparam RR00507 = 8'h00;
localparam RR00508 = 8'h00;
localparam RR00509 = 8'h00;//Branch Device Hardware Revision
localparam RR0050A = 8'h00;//Branch Device Firmware/Software Major Revision
localparam RR0050B = 8'h00;//Branch Device Firmware/Software Minor Revision

localparam RR02002 = 8'h00;//MST mode, SINK_COUNT[7,5:0]=1, CP_READY[6]=1, support HDCP
localparam RR02211 = 8'h00;//DPRX_SLEEP_WAKE_TIMEOUT_PERIOD[7:0]=00h=1ms
localparam RR68000 = 8'h00;
localparam RR68001 = 8'h00;
localparam RR68002 = 8'h00;
localparam RR68003 = 8'h00;
localparam RR68004 = 8'h00;
localparam RR68028 = 8'h00;
localparam RR6921D = 8'h00;
localparam RR6921E = 8'h00;
localparam RR6921F = 8'h00;

localparam RRF0000 = 8'h00;//LT-tunable PHY Repeater DPCD_REV[7:0]
localparam RRF0001 = 8'h00;//MAX_LINK_RATE_PHY_REPEATER
localparam RRF0002 = 8'h00;//PHY_REPEATER_CNT
localparam RRF0003 = 8'h00;//PHY_REPEATER_MODE
localparam RRF0004 = 8'h00;//Repeater_FEC_CAPABILITY
localparam RRF0005 = 8'h00;//RESERVED
localparam RRF0006 = 8'h00;//RESERVED
localparam RRF0007 = 8'h00;//RESERVED

reg  [7:0] rr00202;//LANE0_1_STATUS
reg  [7:0] rr00203;//LANE2_3_STATUS
reg  [7:0] rr00204;//LANE_ALIGN__STATUS_UPDATED
reg  [7:0] rr00205;//SINK_STATUS
reg  [7:0] rr00206;//ADJUST_REQUEST_LANE0_1
reg  [7:0] rr00207;//ADJUST_REQUEST_LANE2_3

//-------------------------------
//Write/Read registers
reg  [7:0] wr00100;//set MAX_LINK_RATE[7:0], 0x14=5.4Gbps/lane (=0.27*RR00001)
reg  [7:0] wr00101;//set LANE_COUNT_SET[4:0],0x04=4 lanes, ENHANCED_FRAME_EN[7], 0=not support
reg  [7:0] wr00102;//TRAINING_PATTERN_SELECT[3:0], 0x1=TPS1, 0x2=TPS2
reg  [7:0] wr00103;//TRAINING_LANE0_SET
reg  [7:0] wr00104;//TRAINING_LANE1_SET
reg  [7:0] wr00105;//TRAINING_LANE2_SET
reg  [7:0] wr00106;//TRAINING_LANE3_SET
reg  [7:0] wr00107;//DOWNSPREAD_CTRL, MSA_TIMING_PAR_IGNORE_EN[7]
reg  [7:0] wr00111;//MST_EN[0]=0, Single Stream format, UPSTREAM_IS_SRC[2]=1, upstream is a source device
reg  [7:0] wr00201;//DEVICE_SERVICE_IRQ_VECTOR, Write 1 to Clear/Read
reg  [7:0] wr00300;//00300h~00302h, Source Device IEEE_OUI
reg  [7:0] wr00301;
reg  [7:0] wr00302;
reg  [7:0] wr00303;//00303h~00308h, Source Device Identification String
reg  [7:0] wr00304;
reg  [7:0] wr00305;
reg  [7:0] wr00306;
reg  [7:0] wr00307;
reg  [7:0] wr00308;
reg  [7:0] wr00309;//Source Device Hardware Revision
reg  [7:0] wr0030a;//Source Device Firmware/Software Major Revision
reg  [7:0] wr0030b;//Source Device Firmware/Software Minor Revision
reg  [7:0] wr00310;//RESERVED
reg  [7:0] wr00311;
reg  [7:0] wr00312;
reg  [7:0] wr00313;
reg  [7:0] wr00314;
reg  [7:0] wr00315;

reg  [7:0] wr00600;//SET_POWER_STATE[2:0],1=enter D0(normal), 2=enter D3(power-down)

reg  [7:0] i2cr_00050;

//===========================================================
//write
reg  [19:0] wr_addr;
reg  [7:0]  wr_len;

//read
reg  [3:0]  r202_cnt;

reg  [19:0] edid_addr;

//for test
reg  [15:0] test_cnt;

//===========================================================
//normal write
always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        wr_addr <= 20'd0;
    else if(I_rx_data_st)
        wr_addr <= I_rx_addr;
    else if(I_rx_data_vld)
        wr_addr <= wr_addr + 1'b1;
    else
        wr_addr <= wr_addr;
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        wr_len <= 9'd0;
    else if(I_rx_data_st)
        wr_len <= 9'd0;
    else if(I_rx_data_vld)
        wr_len <= wr_len + 1'b1;
    else
        wr_len <= wr_len;
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        begin
            wr00100 <= RR00001;
            wr00101 <= RR00002;
            wr00102 <= 8'h00;
            wr00103 <= 8'h00;
            wr00104 <= 8'h00;
            wr00105 <= 8'h00;
            wr00106 <= 8'h00;
            wr00107 <= 8'h00;
            wr00111 <= 8'h04;
            wr00201 <= 8'h00;
            wr00300 <= 8'h00;
            wr00301 <= 8'h00;
            wr00302 <= 8'h00;
            wr00303 <= 8'h00;
            wr00304 <= 8'h00;
            wr00305 <= 8'h00;
            wr00306 <= 8'h00;
            wr00307 <= 8'h00;
            wr00308 <= 8'h00;
            wr00309 <= 8'h00;
            wr0030a <= 8'h00;
            wr0030b <= 8'h00;
            wr00310 <= 8'h00;
            wr00311 <= 8'h00;
            wr00312 <= 8'h00;
            wr00313 <= 8'h00;
            wr00314 <= 8'h00;
            wr00315 <= 8'h00;
            wr00600 <= 8'h01;
        end
    else if(I_rx_data_vld && I_rx_cmd==CMDTYPE_WR)
        begin
            case(wr_addr)
                20'h00100 : wr00100 <= I_rx_data;
                20'h00101 : wr00101 <= I_rx_data;
                20'h00102 : wr00102 <= I_rx_data;
                20'h00103 : wr00103 <= I_rx_data;
                20'h00104 : wr00104 <= I_rx_data;
                20'h00105 : wr00105 <= I_rx_data;
                20'h00106 : wr00106 <= I_rx_data;
                20'h00107 : wr00107 <= I_rx_data;
                20'h00111 : wr00111 <= I_rx_data;
                20'h00201 : wr00201 <= I_rx_data;
                20'h00300 : wr00300 <= I_rx_data;
                20'h00301 : wr00301 <= I_rx_data;
                20'h00302 : wr00302 <= I_rx_data;
                20'h00303 : wr00303 <= I_rx_data;
                20'h00304 : wr00304 <= I_rx_data;
                20'h00305 : wr00305 <= I_rx_data;
                20'h00306 : wr00306 <= I_rx_data;
                20'h00307 : wr00307 <= I_rx_data;
                20'h00308 : wr00308 <= I_rx_data;
                20'h00309 : wr00309 <= I_rx_data;
                20'h0030A : wr0030a <= I_rx_data;
                20'h0030B : wr0030b <= I_rx_data;
                20'h00310 : wr00310 <= I_rx_data;
                20'h00311 : wr00311 <= I_rx_data;
                20'h00312 : wr00312 <= I_rx_data;
                20'h00313 : wr00313 <= I_rx_data;
                20'h00314 : wr00314 <= I_rx_data;
                20'h00315 : wr00315 <= I_rx_data;
                20'h00600 : wr00600 <= I_rx_data;
            endcase
        end
    else
        begin
            wr00100 <= wr00100;
            wr00101 <= wr00101;
            wr00102 <= wr00102;
            wr00103 <= wr00103;
            wr00104 <= wr00104;
            wr00105 <= wr00105;
            wr00106 <= wr00106;
            wr00107 <= wr00107;
            wr00111 <= wr00111;
            wr00201 <= wr00201;
            wr00300 <= wr00300;
            wr00301 <= wr00301;
            wr00302 <= wr00302;
            wr00303 <= wr00303;
            wr00304 <= wr00304;
            wr00305 <= wr00305;
            wr00306 <= wr00306;
            wr00307 <= wr00307;
            wr00308 <= wr00308;
            wr00309 <= wr00309;
            wr0030a <= wr0030a;
            wr0030b <= wr0030b;
            wr00310 <= wr00310;
            wr00311 <= wr00311;
            wr00312 <= wr00312;
            wr00313 <= wr00313;
            wr00314 <= wr00314;
            wr00315 <= wr00315;
            wr00600 <= wr00600;
        end
end

//I2c write
always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        begin
            i2cr_00050 <= 8'h00;
        end
    else if(I_rx_data_vld && I_rx_cmd==CMDTYPE_I2CWRM && I_rx_len!=8'h7f)
        begin
            case(wr_addr)
                20'h00050 : i2cr_00050 <= I_rx_data;
            endcase
        end
    else
        begin
            i2cr_00050 <= i2cr_00050;
        end
end

//===========================================================
//ACK and Read 
//--------------------------
//read data
always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        begin
            O_rx_rply_data <= 128'd0;
        end
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_RD)
        begin
            case(I_rx_addr)
                20'h00000 : O_rx_rply_data <= {RR00000,RR00001,RR00002,RR00003,RR00004,RR00005,RR00006,RR00007,RR00008,RR00009,RR0000A,RR0000B,RR0000C,RR0000D,RR0000E,RR0000F};
                20'h00001 : O_rx_rply_data <= {RR00001,RR00002,RR00003,RR00004,RR00005,RR00006,RR00007,RR00008,RR00009,RR0000A,RR0000B,RR0000C,RR0000D,RR0000E,RR0000F,8'h00  };
                20'h00002 : O_rx_rply_data <= {RR00002,RR00003,RR00004,RR00005,RR00006,RR00007,RR00008,RR00009,RR0000A,RR0000B,RR0000C,RR0000D,RR0000E,RR0000F,8'h00  ,8'h00  };
                20'h00003 : O_rx_rply_data <= {RR00003,RR00004,RR00005,RR00006,RR00007,RR00008,RR00009,RR0000A,RR0000B,RR0000C,RR0000D,RR0000E,RR0000F,8'h00  ,8'h00  ,8'h00  };
                20'h0000E : O_rx_rply_data <= {RR0000E,RR0000F,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00021 : O_rx_rply_data <= {RR00021,RR00022,RR00023,RR00024,RR00025,RR00026,RR00027,RR00028,RR00029,RR0002A,RR0002B,RR0002C,RR0002D,8'h00  ,8'h00  ,8'h00  };
                20'h00023 : O_rx_rply_data <= {RR00023,RR00024,RR00025,RR00026,RR00027,RR00028,RR00029,RR0002A,RR0002B,RR0002C,RR0002D,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00080 : O_rx_rply_data <= {RR00080,RR00081,RR00082,RR00083,RR00084,RR00085,RR00086,RR00087,RR00088,RR00089,RR0008A,RR0008B,RR0008C,RR0008D,RR0008E,RR0008F};
                20'h00100 : O_rx_rply_data <= {wr00100,wr00101,wr00102,wr00103,wr00104,wr00105,wr00106,wr00107,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00101 : O_rx_rply_data <= {wr00101,wr00102,wr00103,wr00104,wr00105,wr00106,wr00107,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00102 : O_rx_rply_data <= {wr00102,wr00103,wr00104,wr00105,wr00106,wr00107,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00103 : O_rx_rply_data <= {wr00103,wr00104,wr00105,wr00106,wr00107,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00104 : O_rx_rply_data <= {wr00104,wr00105,wr00106,wr00107,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00105 : O_rx_rply_data <= {wr00105,wr00106,wr00107,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00106 : O_rx_rply_data <= {wr00106,wr00107,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00107 : O_rx_rply_data <= {wr00107,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00111 : O_rx_rply_data <= {wr00111,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00200 : O_rx_rply_data <= {RR00200,wr00201,rr00202,rr00203,rr00204,rr00205,rr00206,rr00207,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00201 : O_rx_rply_data <= {wr00201,rr00202,rr00203,rr00204,rr00205,rr00206,rr00207,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00202 : O_rx_rply_data <= {rr00202,rr00203,rr00204,rr00205,rr00206,rr00207,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00203 : O_rx_rply_data <= {rr00203,rr00204,rr00205,rr00206,rr00207,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00204 : O_rx_rply_data <= {rr00204,rr00205,rr00206,rr00207,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00206 : O_rx_rply_data <= {rr00206,rr00207,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00207 : O_rx_rply_data <= {rr00207,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00300 : O_rx_rply_data <= {wr00300,wr00301,wr00302,wr00303,wr00304,wr00305,wr00306,wr00307,wr00308,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00301 : O_rx_rply_data <= {wr00301,wr00302,wr00303,wr00304,wr00305,wr00306,wr00307,wr00308,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00302 : O_rx_rply_data <= {wr00302,wr00303,wr00304,wr00305,wr00306,wr00307,wr00308,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00310 : O_rx_rply_data <= {wr00310,wr00311,wr00312,wr00313,wr00314,wr00315,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00400 : O_rx_rply_data <= {RR00400,RR00401,RR00402,RR00403,RR00404,RR00405,RR00406,RR00407,RR00408,RR00409,RR0040A,RR0040B,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00409 : O_rx_rply_data <= {RR00409,RR0040A,RR0040B,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00500 : O_rx_rply_data <= {RR00500,RR00501,RR00502,RR00503,RR00504,RR00505,RR00506,RR00507,RR00508,RR00509,RR0050A,RR0050B,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00509 : O_rx_rply_data <= {RR00509,RR0050A,RR0050B,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h00600 : O_rx_rply_data <= {wr00600,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h68000 : O_rx_rply_data <= {RR68000,RR68001,RR68002,RR68003,RR68004,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h68028 : O_rx_rply_data <= {RR68028,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'h6921D : O_rx_rply_data <= {RR6921D,RR6921E,RR6921F,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                20'hF0000 : O_rx_rply_data <= {RRF0000,RRF0001,RRF0002,RRF0003,RRF0004,RRF0005,RRF0006,RRF0007,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  ,8'h00  };
                default   : O_rx_rply_data <= 128'd0;
            endcase
        end
    else if(I_rx_rply_st && (I_rx_cmd==CMDTYPE_I2CRDM || I_rx_cmd==CMDTYPE_I2CRD))
        begin
            case(edid_addr)
                20'h00000 : O_rx_rply_data <= {EDID_00,EDID_01,EDID_02,EDID_03,EDID_04,EDID_05,EDID_06,EDID_07,EDID_08,EDID_09,EDID_0A,EDID_0B,EDID_0C,EDID_0D,EDID_0E,EDID_0F};
                20'h00010 : O_rx_rply_data <= {EDID_10,EDID_11,EDID_12,EDID_13,EDID_14,EDID_15,EDID_16,EDID_17,EDID_18,EDID_19,EDID_1A,EDID_1B,EDID_1C,EDID_1D,EDID_1E,EDID_1F};
                20'h00020 : O_rx_rply_data <= {EDID_20,EDID_21,EDID_22,EDID_23,EDID_24,EDID_25,EDID_26,EDID_27,EDID_28,EDID_29,EDID_2A,EDID_2B,EDID_2C,EDID_2D,EDID_2E,EDID_2F};
                20'h00030 : O_rx_rply_data <= {EDID_30,EDID_31,EDID_32,EDID_33,EDID_34,EDID_35,EDID_36,EDID_37,EDID_38,EDID_39,EDID_3A,EDID_3B,EDID_3C,EDID_3D,EDID_3E,EDID_3F};
                20'h00040 : O_rx_rply_data <= {EDID_40,EDID_41,EDID_42,EDID_43,EDID_44,EDID_45,EDID_46,EDID_47,EDID_48,EDID_49,EDID_4A,EDID_4B,EDID_4C,EDID_4D,EDID_4E,EDID_4F};
                20'h00050 : O_rx_rply_data <= {EDID_50,EDID_51,EDID_52,EDID_53,EDID_54,EDID_55,EDID_56,EDID_57,EDID_58,EDID_59,EDID_5A,EDID_5B,EDID_5C,EDID_5D,EDID_5E,EDID_5F};
                20'h00060 : O_rx_rply_data <= {EDID_60,EDID_61,EDID_62,EDID_63,EDID_64,EDID_65,EDID_66,EDID_67,EDID_68,EDID_69,EDID_6A,EDID_6B,EDID_6C,EDID_6D,EDID_6E,EDID_6F};
                20'h00070 : O_rx_rply_data <= {EDID_70,EDID_71,EDID_72,EDID_73,EDID_74,EDID_75,EDID_76,EDID_77,EDID_78,EDID_79,EDID_7A,EDID_7B,EDID_7C,EDID_7D,EDID_7E,EDID_7F};
                20'h00080 : O_rx_rply_data <= {EDID_80,EDID_81,EDID_82,EDID_83,EDID_84,EDID_85,EDID_86,EDID_87,EDID_88,EDID_89,EDID_8A,EDID_8B,EDID_8C,EDID_8D,EDID_8E,EDID_8F};
                20'h00090 : O_rx_rply_data <= {EDID_90,EDID_91,EDID_92,EDID_93,EDID_94,EDID_95,EDID_96,EDID_97,EDID_98,EDID_99,EDID_9A,EDID_9B,EDID_9C,EDID_9D,EDID_9E,EDID_9F};
                20'h000A0 : O_rx_rply_data <= {EDID_A0,EDID_A1,EDID_A2,EDID_A3,EDID_A4,EDID_A5,EDID_A6,EDID_A7,EDID_A8,EDID_A9,EDID_AA,EDID_AB,EDID_AC,EDID_AD,EDID_AE,EDID_AF};
                20'h000B0 : O_rx_rply_data <= {EDID_B0,EDID_B1,EDID_B2,EDID_B3,EDID_B4,EDID_B5,EDID_B6,EDID_B7,EDID_B8,EDID_B9,EDID_BA,EDID_BB,EDID_BC,EDID_BD,EDID_BE,EDID_BF};
                20'h000C0 : O_rx_rply_data <= {EDID_C0,EDID_C1,EDID_C2,EDID_C3,EDID_C4,EDID_C5,EDID_C6,EDID_C7,EDID_C8,EDID_C9,EDID_CA,EDID_CB,EDID_CC,EDID_CD,EDID_CE,EDID_CF};
                20'h000D0 : O_rx_rply_data <= {EDID_D0,EDID_D1,EDID_D2,EDID_D3,EDID_D4,EDID_D5,EDID_D6,EDID_D7,EDID_D8,EDID_D9,EDID_DA,EDID_DB,EDID_DC,EDID_DD,EDID_DE,EDID_DF};
                20'h000E0 : O_rx_rply_data <= {EDID_E0,EDID_E1,EDID_E2,EDID_E3,EDID_E4,EDID_E5,EDID_E6,EDID_E7,EDID_E8,EDID_E9,EDID_EA,EDID_EB,EDID_EC,EDID_ED,EDID_EE,EDID_EF};
                20'h000F0 : O_rx_rply_data <= {EDID_F0,EDID_F1,EDID_F2,EDID_F3,EDID_F4,EDID_F5,EDID_F6,EDID_F7,EDID_F8,EDID_F9,EDID_FA,EDID_FB,EDID_FC,EDID_FD,EDID_FE,EDID_FF};
                default   : O_rx_rply_data <= 128'd0;
            endcase
        end
    else
        begin
            O_rx_rply_data <= O_rx_rply_data;
        end
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_rx_rply_ack <= 8'h00;
    else if(I_rx_rply_st) //for test, always reply ACK(00)
        begin
            // if(I_rx_cmd==CMDTYPE_WR && (wr_len!=(I_rx_len+1'b1)))
                // O_rx_rply_ack <= 8'h10;
            // else
            if((I_rx_cmd==CMDTYPE_I2CWRM || I_rx_cmd==CMDTYPE_I2CWR || 
                I_rx_cmd==CMDTYPE_I2CRDM || I_rx_cmd==CMDTYPE_I2CRD) && 
                (I_rx_addr!=20'h00050))
                O_rx_rply_ack <= 8'h40;//I2C NACK(40)
            else
                O_rx_rply_ack <= 8'h00;//ACK(00)
        end
    else
        O_rx_rply_ack <= O_rx_rply_ack;
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_rx_rply_len <= 8'd0;
    else if(I_rx_rply_st)
        O_rx_rply_len <= I_rx_len;
    else
        O_rx_rply_len <= O_rx_rply_len;
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_rx_rply_m <= 8'd0;
    else if(I_rx_rply_st)
        begin
            if(I_rx_cmd==CMDTYPE_WR && (wr_len!=(I_rx_len+1'b1)))
                O_rx_rply_m <= wr_len-1'b1;
            else
                O_rx_rply_m <= 8'd0;
        end
    else
        O_rx_rply_m <= O_rx_rply_m;
end

//----------------------------------------------
//for test, read address 00202 registers counter and 00202~00206 registers change value
always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        r202_cnt <= 4'd0;
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_WR && I_rx_addr==20'h00600)
        r202_cnt <= 4'd0;
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_RD && I_rx_addr==20'h00202)
        r202_cnt <= r202_cnt + 1'b1;
    else
        r202_cnt <= r202_cnt;
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        begin
            rr00202 <= 8'h00;
            rr00203 <= 8'h00;
            rr00204 <= 8'h80;
        end
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_WR && I_rx_addr==20'h00600)
        begin
            rr00202 <= 8'h00;
            rr00203 <= 8'h00;
            rr00204 <= 8'h80;
        end
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_RD && I_rx_addr==20'h00202 && r202_cnt==4'd0)
        begin
            rr00202 <= 8'h11;
            rr00203 <= 8'h11;
            rr00204 <= 8'h80;
        end
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_RD && I_rx_addr==20'h00202 && r202_cnt==4'd1)
        begin
            rr00202 <= 8'h77;
            rr00203 <= 8'h77;
            rr00204 <= 8'h81;
        end
    else
        begin
            rr00202 <= rr00202;
            rr00203 <= rr00203;
            rr00204 <= rr00204;
        end
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        begin
            rr00206 <= 8'h00;
            rr00207 <= 8'h00;
        end
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_WR && I_rx_addr==20'h00600)
        begin
            rr00206 <= 8'h11;
            rr00207 <= 8'h11;
        end
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_RD && I_rx_addr==20'h00202 && r202_cnt==4'd0)
        begin
            rr00206 <= 8'h22;
            rr00207 <= 8'h22;
        end
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_RD && I_rx_addr==20'h00202 && r202_cnt==4'd1)
        begin
            rr00206 <= 8'h66;
            rr00207 <= 8'h66;
        end
    else
        begin
            rr00206 <= rr00206;
            rr00207 <= rr00207;
        end
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        rr00205 <= 8'h00;
    else
        rr00205 <= rr00205;
end

//----------------------------------------------
//EDID addr counter
always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        edid_addr <= 20'd0;
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_I2CWRM && I_rx_len!=8'h7f && I_rx_addr==20'h00050)
        edid_addr <= {12'd0,i2cr_00050};
    else if(I_rx_rply_st && (I_rx_cmd==CMDTYPE_I2CRDM || I_rx_cmd==CMDTYPE_I2CRD) && I_rx_len!=8'h7f)
        edid_addr <= edid_addr + I_rx_len + 1'b1;
    else
        edid_addr <= edid_addr;
end

//=========================================================
always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_rx_tps1_en <= 1'b0;
    else if(wr00102[3:0]==4'h1)
        O_rx_tps1_en <= 1'b1;
    else
        O_rx_tps1_en <= 1'b0;
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_rx_tps2_en <= 1'b0;
    else if(wr00102[3:0]==4'h2)
        O_rx_tps2_en <= 1'b1;
    else
        O_rx_tps2_en <= 1'b0;
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_rx_end <= 1'b0;
    else if(I_rx_rply_st && I_rx_cmd==CMDTYPE_WR && I_rx_addr==20'h00600)
        O_rx_end <= 1'b0;
    else if(I_rx_rply_st && I_rx_addr==20'h00102 && wr00102 == 8'h00)
        O_rx_end <= 1'b1;
    else
        O_rx_end <= O_rx_end;
end

//=========================================================
assign O_frame_type = RR00002[7];//0:normal_frame  1:enhanced_frame
assign O_scrm_value = RR0000D[0];//0:DP rst value 0xffff  1:eDP rst value 0xfffe

//--------------------------------------------
//for test
always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        test_cnt <= 16'd0;
    else if(I_rx_rply_st)
        test_cnt <= test_cnt + 1'd1;
    else
        test_cnt <= test_cnt;
end

endmodule

