
module aux_tx #
(
    parameter FREQ = 20
)
(
    output             O_frame_type      ,
    output             O_scrm_value      ,
    output reg         O_tx_end          ,
    output reg         O_tx_tps1_en      ,
    output reg         O_tx_tps2_en      ,
    output reg         O_tx_video_data_en,
    output reg         O_tx_aux_is_ok    ,
    input              I_aux_clk         ,
    input              I_aux_rstn        ,
    input              I_tx_rply_m_vld   ,
    input      [7:0]   I_tx_rply_m       ,
    input              I_tx_rply_ack_vld ,
    input      [7:0]   I_tx_rply_ack     ,//[5:4]active 
    input      [127:0] I_tx_rply_data    ,//high byte first {byte0,byte1,…,byte15} 
    input      [7:0]   I_tx_rply_len     ,
    input              I_tx_rply_data_vld, 
    output reg         O_tx_st           , 
    output reg         O_tx_i2c_nolen    ,     
    output reg [3:0]   O_tx_cmd          ,  
    output reg [19:0]  O_tx_addr         ,  
    output reg [7:0]   O_tx_len          ,  
    output reg [127:0] O_tx_data          //high byte first {byte0,byte1,…,byte15} 
    
);

localparam WRREG00100 = 8'h06; //MAX_LINK_RATE=0.27*WRREG00100(Gbps)
localparam WRREG00101 = 8'h84; //ENHANCED_FRAME_EN[7], LANE_COUNT_SET[4:0]


localparam TIME250MS = 2500*FREQ; //250000~250ms
localparam TIME670US = 670*FREQ; 

localparam CMDTYPE_WR     = 4'b1000;//8
localparam CMDTYPE_RD     = 4'b1001;//9
localparam CMDTYPE_I2CWRM = 4'b0100;//4
localparam CMDTYPE_I2CRDM = 4'b0101;//5
localparam CMDTYPE_I2CWR  = 4'b0000;//0
localparam CMDTYPE_I2CRD  = 4'b0001;//1

localparam S_IDEL    = 0 ; 
localparam S_LAUNCH  = 1 ;
localparam S_JUDGE   = 2 ;
localparam S_NORMAL  = 3 ;
localparam S_END     = 4 ;

reg[2:0]c_state; 
reg[23:0]cnt_fsm; 
wire start_sync;

//--------------------------
localparam[2:0]IDLE     = 3'd0; 
localparam[2:0]WR   	= 3'd1;
localparam[2:0]RD   	= 3'd2;
localparam[2:0]END      = 3'd3;

reg [2:0]state; 

reg [7:0] rdreg00000;
reg [7:0] rdreg00001;
reg [7:0] rdreg00002;
reg [7:0] rdreg00003;
reg [7:0] rdreg00004;
reg [7:0] rdreg00005;
reg [7:0] rdreg00006;
reg [7:0] rdreg00007;
reg [7:0] rdreg00008;
reg [7:0] rdreg00009;
reg [7:0] rdreg0000a;
reg [7:0] rdreg0000b;
reg [7:0] rdreg0000c;
reg [7:0] rdreg0000d;
reg [7:0] rdreg0000e;
reg [7:0] rdreg0000f;

//------------------------------------
wire start_now  = cnt_fsm == TIME250MS && c_state ==S_IDEL ; //wait 250ms
wire normal_l   = cnt_fsm == TIME670US && c_state == S_NORMAL ; //Interval 670us

////--------------------------------------
localparam  NUM = 55; 
reg [7:0]cnt;  //the max number of cnt must > NUM

wire [127:0]data_mem[0:NUM];
wire [4:0]cmd_mem [0:NUM];
wire [19:0]addr_mem[0:NUM];
wire [7:0]len_mem[0:NUM];

assign cmd_mem[0 ] = 5'h08; assign addr_mem[0 ] = 20'h00600; assign len_mem[0 ] = 8'd0 ; assign data_mem [0 ] =  {8'h01,120'h0};
assign cmd_mem[1 ] = 5'h09; assign addr_mem[1 ] = 20'h00200; assign len_mem[1 ] = 8'd0 ; assign data_mem [1 ] =  128'h0;
assign cmd_mem[2 ] = 5'h09; assign addr_mem[2 ] = 20'h00000; assign len_mem[2 ] = 8'd0 ; assign data_mem [2 ] =  128'h0;
assign cmd_mem[3 ] = 5'h09; assign addr_mem[3 ] = 20'h00500; assign len_mem[3 ] = 8'd10; assign data_mem [3 ] =  128'h0;
assign cmd_mem[4 ] = 5'h09; assign addr_mem[4 ] = 20'h00600; assign len_mem[4 ] = 8'd0 ; assign data_mem [4 ] =  128'h0;
assign cmd_mem[5 ] = 5'h08; assign addr_mem[5 ] = 20'h00600; assign len_mem[5 ] = 8'd0 ; assign data_mem [5 ] =  {8'h01,120'h0};
assign cmd_mem[6 ] = 5'h08; assign addr_mem[6 ] = 20'h00300; assign len_mem[6 ] = 8'd0 ; assign data_mem [6 ] =  {8'h00,120'h0};
assign cmd_mem[7 ] = 5'h08; assign addr_mem[7 ] = 20'h00301; assign len_mem[7 ] = 8'd0 ; assign data_mem [7 ] =  {8'hAA,120'h0};
assign cmd_mem[8 ] = 5'h08; assign addr_mem[8 ] = 20'h00302; assign len_mem[8 ] = 8'd0 ; assign data_mem [8 ] =  {8'h01,120'h0};
assign cmd_mem[9 ] = 5'h09; assign addr_mem[9 ] = 20'h0000E; assign len_mem[9 ] = 8'd0 ; assign data_mem [9 ] =  128'h0;
assign cmd_mem[10] = 5'h09; assign addr_mem[10] = 20'h00000; assign len_mem[10] = 8'd15; assign data_mem [10] =  128'h0;
assign cmd_mem[11] = 5'h09; assign addr_mem[11] = 20'h00080; assign len_mem[11] = 8'd3 ; assign data_mem [11] =  128'h0;
assign cmd_mem[12] = 5'h14; assign addr_mem[12] = 20'h00050; assign len_mem[12] = 8'd0 ; assign data_mem [12] =  128'h0;
assign cmd_mem[13] = 5'h04; assign addr_mem[13] = 20'h00050; assign len_mem[13] = 8'd0 ; assign data_mem [13] =  {8'h00,120'h0};
assign cmd_mem[14] = 5'h15; assign addr_mem[14] = 20'h00050; assign len_mem[14] = 8'd0 ; assign data_mem [14] =  128'h0;
assign cmd_mem[15] = 5'h05; assign addr_mem[15] = 20'h00050; assign len_mem[15] = 8'd3 ; assign data_mem [15] =  128'h0;
assign cmd_mem[16] = 5'h11; assign addr_mem[16] = 20'h00050; assign len_mem[16] = 8'd0 ; assign data_mem [16] =  128'h0;
assign cmd_mem[17] = 5'h14; assign addr_mem[17] = 20'h00050; assign len_mem[17] = 8'd0 ; assign data_mem [17] =  128'h0;
assign cmd_mem[18] = 5'h04; assign addr_mem[18] = 20'h00050; assign len_mem[18] = 8'd0 ; assign data_mem [18] =  {8'h00,120'h0};
assign cmd_mem[19] = 5'h15; assign addr_mem[19] = 20'h00050; assign len_mem[19] = 8'd0 ; assign data_mem [19] =  128'h0;
assign cmd_mem[20] = 5'h05; assign addr_mem[20] = 20'h00050; assign len_mem[20] = 8'd3 ; assign data_mem [20] =  128'h0;
assign cmd_mem[21] = 5'h11; assign addr_mem[21] = 20'h00050; assign len_mem[21] = 8'd0 ; assign data_mem [21] =  128'h0;
assign cmd_mem[22] = 5'h14; assign addr_mem[22] = 20'h00050; assign len_mem[22] = 8'd0 ; assign data_mem [22] =  128'h0;
assign cmd_mem[23] = 5'h04; assign addr_mem[23] = 20'h00050; assign len_mem[23] = 8'd0 ; assign data_mem [23] =  {8'h00,120'h0};
assign cmd_mem[24] = 5'h15; assign addr_mem[24] = 20'h00050; assign len_mem[24] = 8'd0 ; assign data_mem [24] =  128'h0;
assign cmd_mem[25] = 5'h05; assign addr_mem[25] = 20'h00050; assign len_mem[25] = 8'd15; assign data_mem [25] =  128'h0;
assign cmd_mem[26] = 5'h05; assign addr_mem[26] = 20'h00050; assign len_mem[26] = 8'd15; assign data_mem [26] =  128'h0;
assign cmd_mem[27] = 5'h05; assign addr_mem[27] = 20'h00050; assign len_mem[27] = 8'd15; assign data_mem [27] =  128'h0;
assign cmd_mem[28] = 5'h05; assign addr_mem[28] = 20'h00050; assign len_mem[28] = 8'd15; assign data_mem [28] =  128'h0;
assign cmd_mem[29] = 5'h05; assign addr_mem[29] = 20'h00050; assign len_mem[29] = 8'd15; assign data_mem [29] =  128'h0;
assign cmd_mem[30] = 5'h05; assign addr_mem[30] = 20'h00050; assign len_mem[30] = 8'd15; assign data_mem [30] =  128'h0;
assign cmd_mem[31] = 5'h05; assign addr_mem[31] = 20'h00050; assign len_mem[31] = 8'd15; assign data_mem [31] =  128'h0;
assign cmd_mem[32] = 5'h05; assign addr_mem[32] = 20'h00050; assign len_mem[32] = 8'd15; assign data_mem [32] =  128'h0;
assign cmd_mem[33] = 5'h11; assign addr_mem[33] = 20'h00050; assign len_mem[33] = 8'd0 ; assign data_mem [33] =  128'h0;
assign cmd_mem[34] = 5'h09; assign addr_mem[34] = 20'h00200; assign len_mem[34] = 8'd0 ; assign data_mem [34] =  128'h0;
assign cmd_mem[35] = 5'h09; assign addr_mem[35] = 20'h00201; assign len_mem[35] = 8'd0 ; assign data_mem [35] =  128'h0;
assign cmd_mem[36] = 5'h09; assign addr_mem[36] = 20'h00600; assign len_mem[36] = 8'd0 ; assign data_mem [36] =  128'h0;
assign cmd_mem[37] = 5'h08; assign addr_mem[37] = 20'h00600; assign len_mem[37] = 8'd0 ; assign data_mem [37] =  {8'h01,120'h0};
assign cmd_mem[38] = 5'h08; assign addr_mem[38] = 20'h00100; assign len_mem[38] = 8'd0 ; assign data_mem [38] =  {WRREG00100,120'h0};
assign cmd_mem[39] = 5'h09; assign addr_mem[39] = 20'h00101; assign len_mem[39] = 8'd0 ; assign data_mem [39] =  128'h0;
assign cmd_mem[40] = 5'h08; assign addr_mem[40] = 20'h00101; assign len_mem[40] = 8'd0 ; assign data_mem [40] =  {WRREG00101,120'h0};
assign cmd_mem[41] = 5'h08; assign addr_mem[41] = 20'h00102; assign len_mem[41] = 8'd0 ; assign data_mem [41] =  {8'h21,120'h0};
assign cmd_mem[42] = 5'h08; assign addr_mem[42] = 20'h00103; assign len_mem[42] = 8'd3 ; assign data_mem [42] =  {8'h00,8'h00,8'h00,8'h00,96'h0};
assign cmd_mem[43] = 5'h09; assign addr_mem[43] = 20'h00202; assign len_mem[43] = 8'd2 ; assign data_mem [43] =  128'h0;
assign cmd_mem[44] = 5'h09; assign addr_mem[44] = 20'h00206; assign len_mem[44] = 8'd0 ; assign data_mem [44] =  128'h0;
assign cmd_mem[45] = 5'h09; assign addr_mem[45] = 20'h00207; assign len_mem[45] = 8'd0 ; assign data_mem [45] =  128'h0;
assign cmd_mem[46] = 5'h08; assign addr_mem[46] = 20'h00103; assign len_mem[46] = 8'd3 ; assign data_mem [46] =  {8'h01,8'h01,8'h01,8'h01,96'h0};
assign cmd_mem[47] = 5'h09; assign addr_mem[47] = 20'h00202; assign len_mem[47] = 8'd2 ; assign data_mem [47] =  128'h0;
assign cmd_mem[48] = 5'h08; assign addr_mem[48] = 20'h00102; assign len_mem[48] = 8'd0 ; assign data_mem [48] =  {8'h22,120'h0};
assign cmd_mem[49] = 5'h08; assign addr_mem[49] = 20'h00103; assign len_mem[49] = 8'd3 ; assign data_mem [49] =  {8'h01,8'h01,8'h01,8'h01,96'h0};
assign cmd_mem[50] = 5'h09; assign addr_mem[50] = 20'h00202; assign len_mem[50] = 8'd2 ; assign data_mem [50] =  128'h0;
assign cmd_mem[51] = 5'h09; assign addr_mem[51] = 20'h00206; assign len_mem[51] = 8'd0 ; assign data_mem [51] =  128'h0;
assign cmd_mem[52] = 5'h09; assign addr_mem[52] = 20'h00207; assign len_mem[52] = 8'd0 ; assign data_mem [52] =  128'h0;
assign cmd_mem[53] = 5'h08; assign addr_mem[53] = 20'h00103; assign len_mem[53] = 8'd3 ; assign data_mem [53] =  {8'h06,8'h06,8'h06,8'h06,96'h0};
assign cmd_mem[54] = 5'h09; assign addr_mem[54] = 20'h00202; assign len_mem[54] = 8'd2 ; assign data_mem [54] =  128'h0;
assign cmd_mem[55] = 5'h08; assign addr_mem[55] = 20'h00102; assign len_mem[55] = 8'd0 ; assign data_mem [55] =  {8'h00,120'h0};

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn) 
        c_state       <= S_IDEL; 
    else case(c_state)
        S_IDEL:
            if(start_now )  
				c_state <=  S_LAUNCH;
		S_LAUNCH : 
			if(cnt >= NUM)
				c_state   <= S_END;
			else
				c_state   <= S_JUDGE;
        S_JUDGE : 
				c_state   <= S_NORMAL;
		S_NORMAL:
			if(normal_l)
				c_state <= S_LAUNCH;
		S_END: ;
        default: 
            c_state       <= S_IDEL;  
    endcase
        
end 
always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        cnt_fsm <= 24'd0;
    else if(start_now || normal_l)
		cnt_fsm <= 24'd0;
	else
        cnt_fsm <= cnt_fsm + 1'b1 ;
end  

//-------------------------------------------------
assign start_sync = c_state ==S_LAUNCH;

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn) 
        state       <= IDLE; 
    else case(state)
        IDLE:
            if(start_sync )  
                begin
					if(cmd_mem[cnt][3:0]==CMDTYPE_WR || 
                       cmd_mem[cnt][3:0]==CMDTYPE_I2CWRM || 
                       cmd_mem[cnt][3:0]==CMDTYPE_I2CWR)
						state <= WR ;
					else  
					    state <= RD ; 
				end				
        WR : 
			state   <= END; 
		RD : 
			state   <= END;
        END : 
            if(I_tx_rply_ack_vld)
                state   <= IDLE; 
            else
                state   <= END;
        default: 
            state   <= IDLE;  
    endcase
        
end  


always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)  
        cnt   <=  8'd0;  
    else if(state==END && I_tx_rply_ack_vld && I_tx_rply_ack[5:4]==2'b00)
        cnt   <=  cnt + 1'b1 ;
end 
 

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_tx_addr    <=  20'd0;
    else if(start_sync)   
        O_tx_addr    <=  addr_mem[cnt]; 
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_tx_len    <=  8'd0;
    else if(start_sync)   
        O_tx_len    <=  len_mem[cnt]; 
end
always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        begin
            O_tx_i2c_nolen <= 1'b0;
            O_tx_cmd       <= 4'd0;
        end
    else if(start_sync)   
        begin
            O_tx_i2c_nolen <= cmd_mem[cnt][4];
            O_tx_cmd       <= cmd_mem[cnt][3:0]; 
        end
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn) 
        O_tx_data   <=  'd0; 
	// else if(state == WR) 
        // O_tx_data  <= data_mem[cnt];
    else if(start_sync)   
        O_tx_data  <= data_mem[cnt]; 
end

always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_tx_st     <=  'd0;
    else  if(state == WR | state==RD)  
        O_tx_st <= 1'b1; 
	else  if(state == END)
		O_tx_st     <= 1'd0;   
end 


always@(posedge I_aux_clk or negedge I_aux_rstn) 
begin
    if(!I_aux_rstn)
        O_tx_end   <=  1'd0;
    else if(cnt==(NUM+1))   
        O_tx_end   <=  1'b1; 
end


//==================================================================================
always@(posedge I_aux_clk or negedge I_aux_rstn)
begin
    if(!I_aux_rstn) 
        O_tx_tps1_en <=  1'd0; 
    else if(cnt==8'd41)
        O_tx_tps1_en <=  1'd1;
    // else if(O_tx_st && O_tx_cmd == 4'd8 && O_tx_addr == 'h102 && O_tx_data == {24'h220101,104'h0})
    else if(cnt==8'd49)
		O_tx_tps1_en <=  1'd0;
    else
        O_tx_tps1_en <=  O_tx_tps1_en; 
end

always@(posedge I_aux_clk or negedge I_aux_rstn)
begin
    if(!I_aux_rstn) 
        O_tx_tps2_en <=  1'd0; 
    else if(cnt==8'd49)
		O_tx_tps2_en <=  1'd1;
    // else if(O_tx_st && O_tx_cmd == 4'd8 && O_tx_addr == 'h102 && O_tx_data == 128'h0 && O_tx_len == 'd0) 
    else if(cnt==8'd56)
		O_tx_tps2_en <=  1'd0;
    else
        O_tx_tps2_en <=  O_tx_tps2_en; 
end

always@(posedge I_aux_clk or negedge I_aux_rstn)
begin
    if(!I_aux_rstn) 
        O_tx_video_data_en <=  1'd0; 
    else if(cnt==8'd56)
		O_tx_video_data_en <=  1'd1; 
    else
        O_tx_video_data_en <=  O_tx_video_data_en; 
end

//--------------------
always@(posedge I_aux_clk or negedge I_aux_rstn)
begin
    if(!I_aux_rstn) 
        O_tx_aux_is_ok <=  1'd0; 
    else if(I_tx_rply_data_vld && I_tx_rply_data[23:8] == {8'h77,8'h77})  
		O_tx_aux_is_ok <=  1'd1;
    else
        O_tx_aux_is_ok <=  O_tx_aux_is_ok; 
end

//===========================================================
always@(posedge I_aux_clk or negedge I_aux_rstn)
begin
    if(!I_aux_rstn) 
        begin
            rdreg00000 <= 8'd0;
            rdreg00001 <= 8'd0;
            rdreg00002 <= 8'd0;
            rdreg00003 <= 8'd0;
            rdreg00004 <= 8'd0;
            rdreg00005 <= 8'd0;
            rdreg00006 <= 8'd0;
            rdreg00007 <= 8'd0;
            rdreg00008 <= 8'd0;
            rdreg00009 <= 8'd0;
            rdreg0000a <= 8'd0;
            rdreg0000b <= 8'd0;
            rdreg0000c <= 8'd0;
            rdreg0000d <= 8'd0;
            rdreg0000e <= 8'd0;
            rdreg0000f <= 8'd0;
        end
    else if(O_tx_cmd == 5'h09 && O_tx_addr == 20'h00000 && O_tx_len == 8'd15 && I_tx_rply_data_vld) 
        begin
            rdreg00000 <= I_tx_rply_data[127:120];
            rdreg00001 <= I_tx_rply_data[119:112];
            rdreg00002 <= I_tx_rply_data[111:104];
            rdreg00003 <= I_tx_rply_data[103: 96];
            rdreg00004 <= I_tx_rply_data[ 95: 88];
            rdreg00005 <= I_tx_rply_data[ 87: 80];
            rdreg00006 <= I_tx_rply_data[ 79: 72];
            rdreg00007 <= I_tx_rply_data[ 71: 64];
            rdreg00008 <= I_tx_rply_data[ 63: 56];
            rdreg00009 <= I_tx_rply_data[ 55: 48];
            rdreg0000a <= I_tx_rply_data[ 47: 40];
            rdreg0000b <= I_tx_rply_data[ 39: 32];
            rdreg0000c <= I_tx_rply_data[ 31: 24];
            rdreg0000d <= I_tx_rply_data[ 23: 16];
            rdreg0000e <= I_tx_rply_data[ 15:  8];
            rdreg0000f <= I_tx_rply_data[  7:  0];
        end
    else
        begin
            rdreg00000 <= rdreg00000;
            rdreg00001 <= rdreg00001;
            rdreg00002 <= rdreg00002;
            rdreg00003 <= rdreg00003;
            rdreg00004 <= rdreg00004;
            rdreg00005 <= rdreg00005;
            rdreg00006 <= rdreg00006;
            rdreg00007 <= rdreg00007;
            rdreg00008 <= rdreg00008;
            rdreg00009 <= rdreg00009;
            rdreg0000a <= rdreg0000a;
            rdreg0000b <= rdreg0000b;
            rdreg0000c <= rdreg0000c;
            rdreg0000d <= rdreg0000d;
            rdreg0000e <= rdreg0000e;
            rdreg0000f <= rdreg0000f;
        end
end

assign O_frame_type = WRREG00101[7];
assign O_scrm_value = rdreg0000d[0];

endmodule
