//---------------------------------------------------------------------
// File name  : tb.v
// Module name: tb
// Created by : 
// ---------------------------------------------------------------------
// Release history
// ----------------------------------------------------------------------------------
// Ver:    |  Author    | Mod. Date    | Changes Made:
// ----------------------------------------------------------------------------------
// V1.0    | Caojie     | 08/17/23     | Initial version 
// ----------------------------------------------------------------------------------

`timescale 1ns / 1ps

module tb();

//========================================================
//parameters
parameter BMP_PIXEL_CLK_PERIOD	   = 26.957; //unit: ns  1000/(148.3872/4)MHz=26.957ns
parameter BMP_LINK                 = 2'd2;  //2'd0,one pixel mode; 2'd1,two pixel mode; 2'd2,four pixel mode
								   
parameter BMP_OUTPUTED_WIDTH	   = 1920;
parameter BMP_OUTPUTED_HEIGHT	   = 1080;
parameter BMP_OUTPUTED_NAME		   = "../../tb/pic/out0_001.bmp";
parameter BMP_OUTPUTED_NUMBER	   = 16'd5;

//---------------------------
parameter LS_CLK_PERIOD	       = 12.346; //unit: ns  1000/81MHz=12.346ns 

//=======================================================
reg  pixel_clk;

reg  ls_clock;

reg  rst_n;

//------------
//dirver
wire        tp0_vs_in  ;
wire        tp0_hs_in  ;
wire        tp0_de_in ;
wire [ 7:0] tp0_data_r;
wire [ 7:0] tp0_data_g;
wire [ 7:0] tp0_data_b;
wire [ 7:0] tp1_data_r;
wire [ 7:0] tp1_data_g;
wire [ 7:0] tp1_data_b;
wire [ 7:0] tp2_data_r;
wire [ 7:0] tp2_data_g;
wire [ 7:0] tp2_data_b;
wire [ 7:0] tp3_data_r;
wire [ 7:0] tp3_data_g;
wire [ 7:0] tp3_data_b;

wire[15:0]I_h_total  ;
wire[15:0]I_h_sync   ;
wire[15:0]I_h_bporch ;
wire[15:0]I_h_res    ;
wire[15:0]I_v_total  ;
wire[15:0]I_v_sync   ;
wire[15:0]I_v_bporch ;
wire[15:0]I_v_res    ;

//--------------------------
wire [3:0]  tx0_kcode;
wire [31:0] tx0_data;
wire [3:0]  rx0_kcode;
wire [31:0] rx0_data;
     
wire [3:0]  tx1_kcode;
wire [31:0] tx1_data;
wire [3:0]  rx1_kcode;
wire [31:0] rx1_data;
     
wire [3:0]  tx2_kcode;
wire [31:0] tx2_data;
wire [3:0]  rx2_kcode;
wire [31:0] rx2_data;
     
wire [3:0]  tx3_kcode;
wire [31:0] tx3_data;
wire [3:0]  rx3_kcode;
wire [31:0] rx3_data;

//-------------------------
wire [7:0]  vb_id          ;
wire [7:0]  mvid70         ;
wire [7:0]  maud70         ;
wire [23:0] mvid           ;//
wire [23:0] nvid           ;//
wire [15:0] htotal         ;//
wire [15:0] vtotal         ;//
wire [15:0] hstart         ;//
wire [15:0] vstart         ;//
wire [15:0] hwidth         ;//
wire [15:0] vheight        ;//
wire [14:0] hsw            ;//
wire [14:0] vsw            ;//
wire        hsp            ;//
wire        vsp            ;//
wire [7:0]  misc0          ;//
wire [7:0]  misc1          ;//
wire        ls_vs          ;
wire        ls_hs          ;
wire [47:0] unp0_data      ;// 
wire        unp0_data_valid;//
wire [47:0] unp1_data      ;//
wire        unp1_data_valid;//
wire [47:0] unp2_data      ;//
wire        unp2_data_valid;//
wire [47:0] unp3_data      ;//
wire        unp3_data_valid;//

//-------------------------
wire        rx_vs   ;
wire        rx_hs   ;
wire        rx_de   ;
wire [7:0]  rx0_r      ; 
wire [7:0]  rx0_g      ; 
wire [7:0]  rx0_b      ; 
wire [7:0]  rx1_r      ; 
wire [7:0]  rx1_g      ; 
wire [7:0]  rx1_b      ;
wire [7:0]  rx2_r      ; 
wire [7:0]  rx2_g      ; 
wire [7:0]  rx2_b      ; 
wire [7:0]  rx3_r      ; 
wire [7:0]  rx3_g      ; 
wire [7:0]  rx3_b      ;


//-----------------
//monitor rgb input
wire		m_clk;
wire		m_vs_rgb;  
wire		m_hs_rgb;  
wire		m_de_rgb;  
wire [7:0]  m_data0_r;
wire [7:0]  m_data0_g;
wire [7:0]  m_data0_b;
wire [7:0]  m_data1_r;
wire [7:0]  m_data1_g;
wire [7:0]  m_data1_b;
wire [7:0]  m_data2_r;
wire [7:0]  m_data2_g;
wire [7:0]  m_data2_b;
wire [7:0]  m_data3_r;
wire [7:0]  m_data3_g;
wire [7:0]  m_data3_b;

//=====================================================
GSR GSR(.GSRI(1'b1));

//==============================================  
initial begin
  $fsdbDumpfile("tb.fsdb");
  $fsdbDumpvars;
end

//=====================================================
initial
  begin
	pixel_clk	     = 1'b0;
    ls_clock	     = 1'b0;
  end

always  #(BMP_PIXEL_CLK_PERIOD/2.0) pixel_clk = ~pixel_clk;
always  #(LS_CLK_PERIOD/2.0) ls_clock = ~ls_clock;


//=====================================================
//rst_n
initial
  begin
	rst_n=1'b0;
	
	#2000;
	rst_n=1'b1;
end

//==================================================
//video driver
testpatternX4 testpatternX4_inst
(
    .I_pxl_clk   (pixel_clk          ),//pixel clock
    .I_rst_n     (rst_n              ),//low active  
    .I_mode      (4'd0               ),//data select
    .I_sqr_width (16'd32             ),
    .I_single_r  (8'd0               ),
    .I_single_g  (8'd255             ),//148.5M(Actually148.3872M)    
    .I_single_b  (8'd0               ),//1920x1080  
    .I_h_total   (16'd550            ),// 16'd2200  
    .I_h_sync    (16'd11             ),// 16'd44    
    .I_h_bporch  (16'd37             ),// 16'd148   
    .I_h_res     (16'd480            ),// 16'd1920  
    .I_v_total   (16'd1125           ),// 16'd1125  
    .I_v_sync    (16'd5              ),// 16'd5     
    .I_v_bporch  (16'd36             ),// 16'd36    
    .I_v_res     (16'd1080           ),// 16'd1080  
    .O_vblank    (                   ),
    .O_hblank    (                   ),  
    .O_vs        (tp0_vs_in          ),//polarity fixed negative
    .O_hs        (tp0_hs_in          ),//polarity fixed negative
    .O_de        (tp0_de_in          ),
    .O_data0_r   (tp0_data_r         ),   
    .O_data0_g   (tp0_data_g         ),
    .O_data0_b   (tp0_data_b         ),
    .O_data1_r   (tp1_data_r         ), 
    .O_data1_g   (tp1_data_g         ),
    .O_data1_b   (tp1_data_b         ),
    .O_data2_r   (tp2_data_r         ), 
    .O_data2_g   (tp2_data_g         ),
    .O_data2_b   (tp2_data_b         ),
    .O_data3_r   (tp3_data_r         ), 
    .O_data3_g   (tp3_data_g         ),
    .O_data3_b   (tp3_data_b         )
);

//======================================================
//EDP TX
EDP_Encoder_Top EDP_Encoder_Top_inst
(
    .I_rst_n           (rst_n             ),
    .I_tx_tps1_en      (1'b0              ),
    .I_tx_tps2_en      (1'b0              ),
    .I_tx_video_data_en(1'b1              ),
    .I_frame_type      (1'b1              ),//0:normal_frame  1:enhanced_frame
    .I_scrm_value      (1'b0              ),//0:DP rst value 0xffff  1:eDP rst value 0xfffe
    .I_vb_id           (8'h10             ),
    .I_msa_mvid        (24'd30012         ), 
    .I_msa_nvid        (24'd32768         ),
    .I_tu_size         (8'd32             ),
    .I_avs_per_tu      (8'd22             ),
    .I_lsck_htotal     (16'd1201          ),
    .I_lsck_hblank     (16'd156           ),
    .I_msa_htotal      (16'd2200          ),
    .I_msa_vtotal      (16'd1125          ),
    .I_msa_hstart      (16'd192           ),
    .I_msa_vstart      (16'd41            ),
    .I_msa_hwidth      (16'd1920          ),
    .I_msa_vheight     (16'd1080          ),
    .I_msa_hsphsw      ({1'b0,15'd44}     ),
    .I_msa_vspvsw      ({1'b0,15'd5}      ),
    .I_msa_misc0       (8'h20             ),
    .I_msa_misc1       (8'h00             ),
    .I_strm_clk        (pixel_clk         ),  
    .I_pix_vs          (~tp0_vs_in        ),//postive ,rising edge trigger
    .I_pix_hs          (~tp0_hs_in        ),//postive ,falling edge trigger
    .I_pix_de          (tp0_de_in         ),
    .I_pix_data0       ({tp0_data_b,tp0_data_g,tp0_data_r}),
    .I_pix_data1       ({tp1_data_b,tp1_data_g,tp1_data_r}), 
    .I_pix_data2       ({tp2_data_b,tp2_data_g,tp2_data_r}),
    .I_pix_data3       ({tp3_data_b,tp3_data_g,tp3_data_r}),
    .I_ls_clk          (ls_clock          ),     
    .O_tx_data_0       (tx0_data          ),    
    .O_tx_data_1       (tx1_data          ), 
    .O_tx_data_2       (tx2_data          ),    
    .O_tx_data_3       (tx3_data          ),      
    .O_k_code_0        (tx0_kcode         ),    
    .O_k_code_1        (tx1_kcode         ),  
    .O_k_code_2        (tx2_kcode         ),    
    .O_k_code_3        (tx3_kcode         )  
);

assign rx0_kcode = tx0_kcode;
assign rx0_data  = tx0_data ;
assign rx1_kcode = tx1_kcode;
assign rx1_data  = tx1_data ;
assign rx2_kcode = tx2_kcode;
assign rx2_data  = tx2_data ;
assign rx3_kcode = tx3_kcode;
assign rx3_data  = tx3_data ;


//======================================================
//EDP RX
EDP_Decoder_Top EDP_Decoder_Top_inst
( 
    .I_rst_n          (rst_n          ),//low active
    .I_ls_clk         (ls_clock       ),
    .I_frame_type     (1'b1           ),//0:normal_frame  1:enhanced_frame
    .I_scrm_value     (1'b0           ),//0:DP rst value 0xffff  1:eDP rst value 0xfffe
    .O_vb_id          (vb_id          ),
    .O_mvid70         (mvid70         ),
    .O_maud70         (maud70         ),
    .O_mvid           (mvid           ),//
    .O_nvid           (nvid           ),//
    .O_htotal         (htotal         ),//
    .O_vtotal         (vtotal         ),//
    .O_hstart         (hstart         ),//
    .O_vstart         (vstart         ),//
    .O_hwidth         (hwidth         ),//
    .O_vheight        (vheight        ),//
    .O_hsw            (hsw            ),//
    .O_vsw            (vsw            ),//
    .O_hsp            (hsp            ),//
    .O_vsp            (vsp            ),//
    .O_misc0          (misc0          ),//
    .O_misc1          (misc1          ),//
    .O_ls_vs          (ls_vs          ),
    .O_ls_hs          (ls_hs          ),
    .I_kcode0         (rx0_kcode      ),//
    .I_data0          (rx0_data       ),//low byte first
    .I_kcode1         (rx1_kcode      ),//
    .I_data1          (rx1_data       ),//
    .I_kcode2         (rx2_kcode      ),//
    .I_data2          (rx2_data       ),//
    .I_kcode3         (rx3_kcode      ),//
    .I_data3          (rx3_data       ),//
    .O_unp0_data      (unp0_data      ),//low byte first
    .O_unp0_data_valid(unp0_data_valid),//
    .O_unp1_data      (unp1_data      ),//
    .O_unp1_data_valid(unp1_data_valid),
    .O_unp2_data      (unp2_data      ),//
    .O_unp2_data_valid(unp2_data_valid),//
    .O_unp3_data      (unp3_data      ),//
    .O_unp3_data_valid(unp3_data_valid)
);

//----------------------------
EDP_RX_Desteer_Top EDP_RX_Desteer_Top_inst
( 
    .I_rst_n          (rst_n          ),//low active
    .I_ls_clk         (ls_clock       ),
    .I_ls_vs          (ls_vs          ),//postive
    .I_ls_hs          (ls_hs          ),//postive
    .I_htotal         (htotal         ),//
    .I_vtotal         (vtotal         ),//
    .I_hstart         (hstart         ),//
    .I_vstart         (vstart         ),//
    .I_hwidth         (hwidth         ),//
    .I_vheight        (vheight        ),//
    .I_hsw            (hsw            ),//
    .I_vsw            (vsw            ),//
    .I_unp0_data      (unp0_data      ),// 
    .I_unp0_data_valid(unp0_data_valid),//
    .I_unp1_data      (unp1_data      ),//
    .I_unp1_data_valid(unp1_data_valid),//
    .I_unp2_data      (unp2_data      ),//
    .I_unp2_data_valid(unp2_data_valid),//
    .I_unp3_data      (unp3_data      ),//
    .I_unp3_data_valid(unp3_data_valid),//
    .I_strm_clk       (pixel_clk      ),//pixel clock
    .O_vcnt_strmclk   (               ),
    .O_hcnt_strmclk   (               ),
    .O_vs             (rx_vs          ),//postive
    .O_hs             (rx_hs          ),//postive
    .O_de             (rx_de          ),//
    .O_data0          ({rx0_b,rx0_g,rx0_r}),//
    .O_data1          ({rx1_b,rx1_g,rx1_r}),//
    .O_data2          ({rx2_b,rx2_g,rx2_r}),//
    .O_data3          ({rx3_b,rx3_g,rx3_r}) //
);

//======================================================
//monitor
assign m_clk     = pixel_clk       ;
assign m_vs_rgb  = rx_vs      ;
assign m_hs_rgb  = rx_hs      ;
assign m_de_rgb  = rx_de         ;
assign m_data0_r = rx0_r		; 
assign m_data0_g = rx0_g		;
assign m_data0_b = rx0_b		;
assign m_data1_r = rx1_r      	;
assign m_data1_g = rx1_g      	;
assign m_data1_b = rx1_b      	;
assign m_data2_r = rx2_r      	;
assign m_data2_g = rx2_g      	;
assign m_data2_b = rx2_b      	;
assign m_data3_r = rx3_r      	;
assign m_data3_g = rx3_g      	;
assign m_data3_b = rx3_b      	;

monitor#
(
  .BMP_OUTPUTED_WIDTH  (BMP_OUTPUTED_WIDTH ),
  .BMP_OUTPUTED_HEIGHT (BMP_OUTPUTED_HEIGHT),
  .BMP_OUTPUTED_NAME   (BMP_OUTPUTED_NAME  ),
  .BMP_OUTPUTED_NUMBER (BMP_OUTPUTED_NUMBER)
)
monitor_inst
(
    .link_i       (BMP_LINK          ), //2'd0,one pixel mode; 2'd1,two pixel mode; 2'd2,four pixel mode
    .video2bmp_en (rst_n             ),
    .pixel_clock  (m_clk             ), 
    .vsync        (m_vs_rgb          ),        
    .hsync        (m_hs_rgb          ),        
    .data_valid   (m_de_rgb          ), 
    .data0_r      (m_data0_r         ),    
    .data0_g      (m_data0_g         ),
    .data0_b      (m_data0_b         ),
    .data1_r      (m_data1_r         ),    
    .data1_g      (m_data1_g         ),
    .data1_b      (m_data1_b         ),
    .data2_r      (m_data2_r         ),    
    .data2_g      (m_data2_g         ),
    .data2_b      (m_data2_b         ),
    .data3_r      (m_data3_r         ),    
    .data3_g      (m_data3_g         ),
    .data3_b      (m_data3_b         )
);
		  
endmodule
